/*
 * tacos.hpp
 *
 *  Created on: Jan 2, 2024
 *      Author: Dario
 */

#ifndef STA_TACOS_HPP
#define STA_TACOS_HPP

#include <sta/tacos/thread.hpp>
#include <sta/tacos/manager.hpp>
#include <sta/tacos/statemachine.hpp>

#include <initializer_list>
#include <type_traits>


namespace sta
{
	namespace tacos
	{
		/**
		 * @brief Get the current state of the TACOS statemachine.
		 * 
		 * @return uint16_t Returns the state.
		 */
		uint16_t getState();

		/**
		 * @brief Request a state transition. Invalid state transitions will be dismissed.
		 * 
		 * @param from The start we want to transition from.
		 * @param to The state we want to transition to.
		 * @param lockout An optional timer blocking state transition for a given time.
		 */
		void setState(uint32_t from, uint32_t to, uint32_t lockout = 0);

		/**
		 * @brief Request a state transition after a given time has passed. Invalid state transitions will be dismissed.
		 * 
		 * @param from The start we want to transition from.
		 * @param to The state we want to transition to.
		 * @param millis The time to wait until the transition is requested.
		 * @param lockout An optional timer blocking state transition for a given time. Will be active after this transition was executed.
		 */
		void setStateTimed(uint32_t from, uint32_t to, uint32_t millis, uint32_t lockout = 0);

		/**
		 * @brief Register a new thread to be run by TACOS.
		 * 
		 * @tparam T The class of the thread to be created. A subclass of TacosThread.
		 * @param states A list of states in which the thread should run. 
		 * @param args The constructor arguments for the provided class.
		 */
		template<typename T, typename ... Args>
		void addThread(std::list<uint16_t> states, Args ... args)
		{
            Manager::instance()->registerThread(std::make_shared<T>(args...), states);
        }
	}  // namespace tacos
}

#endif /* STA_TACOS_HPP */
